﻿import { Basic } from "../basic.js";
import { Converter } from '../../observer/converter.js'
import * as L from "../../lib/leaflet-src-esm.js"

/*<link rel="stylesheet" href="/controls/map/leaflet.css" />
<div id='mapid' class='mapid'></div>*/

//problems:
//reinit markers after map load

export class MSMap extends Basic {

    static get observedAttributes() {
        return super.observedAttributes.concat([
            'paneldatasource',
            'markerstyles',
            'modelrefs',
            'tileserver',
            'tilesubdomains',
            'initpoint',
            'maxzoom',
            'minzoom',
            'maptype'
        ]);
    }

    constructor() {
        super();
        this._mapElement = this.getElementsByClassName("mapid")[0];

        //'http://vec{s}.maps.yandex.net/tiles?l=map&v=4.55.2&z={z}&x={x}&y={y}&scale=2&lang=ru_RU'
        //'https://b.tile.openstreetmap.org/{z}/{x}/{y}.png'        

        //
        this._initpoint = [0, 0]
        this._initzoom = 2;
        this._maxzoom = 23;
        this._minzoom = 0;
        this._maptype = "0";
        this._useindex = true;
        //
        this._markerProps = ["Tooltip", "MarkerType", "X", "Y"];

        this.currentMarkers = [];
        this.inited = false;
    }

    afterInitialize() {
        this._Lmap = L.map(`mapid_${this.id}`, { crs: L.CRS.EPSG3395, }).setView(this._initpoint, this._initzoom);
        this._Lmap.on('click', this.mapOnClick.bind(this));
        if (this._tileserver && Array.isArray(this._tileserver)) {
            if (this._maptype === "0") {
                if (this._tileserver.length === 1) {
                    this.createMainTileMap(this._tileserver[0])
                } else {
                    let tileArray = [...this._tileserver];
                    let main = tileArray.shift();
                    this.createMainTileMap(main);

                    let options = { //create base options? TODO
                        reuseTiles: true,
                        updateWhenIdle: false,
                        maxZoom: this._maxzoom,
                        minZoom: this._minzoom,
                    };
                    tileArray.forEach(url => L.tileLayer(url, options).addTo(this._Lmap));
                }
            } else if (this._maptype === '1') {
                let path = this._tileserver[0];
                let tileResPath = `resources/${path}/{z}/tile-{x}-{y}.png`;
                this.createMainTileMap(tileResPath);
            }
        } else if (this._maptype === "0" && this.hasAttribute("useyandex")) {
            let layerType = this.getAttribute("useyandex");
            this.createYandexLayer(layerType);
        }

        this.inited = true;
    }

    attributeChangedCallback(attrName, oldVal, newVal, itemIndex, sourceAttrName) {
        if (attrName === 'paneldatasource') {
            if (typeof itemIndex !== undefined && typeof sourceAttrName !== "undefined") {

                let paramStruct = this.modelParamsStruct[sourceAttrName];
                let dsField = paramStruct.FieldName;

                let currentValue = this._paneldatasource[itemIndex][dsField];
                let valueConvert = Converter.convert(paramStruct, currentValue, paramStruct.FieldType);
                //исходное значение может быть сконвертировано, 
                //тогда значение в источнике данных может отличаться, при том что изменений не было
                //так проверяем конвертацию сохраненного значения и новое значение                
                if (newVal != valueConvert) {
                    let value = Converter.convert(paramStruct, newVal, paramStruct.FieldType);
                    this._paneldatasource[itemIndex][dsField] = value;
                    if (this.prop_link["paneldatasource"]) {
                        this.prop_link["paneldatasource"](
                            this.pId ? this.pId + '/' + this.id : this.id,
                            this.Links["paneldatasource"],
                            this._paneldatasource,
                            "paneldatasource",
                            itemIndex
                        );
                    }
                }
            } else {
                this.paneldatasource = newVal;
            }
        } else {
            super.attributeChangedCallback(attrName, oldVal, newVal);
        }
    }

    createMainTileMap(url) {
        let options = {
            maxZoom: this._maxzoom,
            minZoom: this._minzoom,
            subdomains: this._tilesubdomains || "",
            reuseTiles: true,
            updateWhenIdle: false,
            tileSize: 512, //use special parameter?? TODO
            zoomOffset: -1,
        }
        L.tileLayer(url, options).addTo(this._Lmap);
    }

    createYandexLayer(type) {
        let layer = type || "map"
        L.yandex(`yandex#${layer}`).addTo(this._Lmap);
    }

    get width() {
        return this._width;
    }

    set width(value) {
        this._width = value;
        const tmp = Number(value);
        if (tmp) {
            this._mapElement.style.width = this._setValueUnit(value);
        }
    }

    get height() {
        return this._height;
    }

    set height(value) {
        this._height = value;
        const tmp = Number(value);
        if (tmp) {
            this._mapElement.style.height = this._setValueUnit(value);
        }
    }

    get modelrefs() {
        return this._modelrefs;
    }

    set modelrefs(value) {
        try {
            var ds = JSON.parse(value);
        } catch (e) {
            ds = {};
        }
        this._modelrefs = ds;
    }

    get markerstyles() {
        return this._markerstyles;
    }

    set markerstyles(value) {
        try {
            var ds = JSON.parse(value);
        } catch (e) {
            ds = {};
        }
        this._markerstyles = ds;

        if (!this.modelParamsStruct) {
            this.calcModelParams(ds);
        }
    }

    get paneldatasource() {
        return this._paneldatasource;
    }

    set paneldatasource(value) {
        if (value && value.value) {
            this._paneldatasource_dataType = value.dataType;
            var ds = value.value;
        } else {
            try {
                ds = JSON.parse(value);
            } catch (ex) {
                ds = []
            }
        }
        this._paneldatasource = ds;
        if (this.inited) {
            this.processDataSource(ds);
        }
    }

    get maptype() {
        return this._maptype;
    }
    set maptype(value) {
        this._maptype = value;
    }

    get tileserver() {
        return this._tileserver;
    }
    set tileserver(value) {
        if (value) {
            let val = value.split(',');
            this._tileserver = val;
        }
    }

    get tilesubdomains() {
        return this._tilesubdomains;
    }
    set tilesubdomains(value) {
        if (value) {
            let val = value.split(',').map(x => x.trim());
            this._tilesubdomains = val;
        }

    }

    get initpoint() {
        return this._initpoint;
    }
    set initpoint(value) {
        let val = [];
        if (value && typeof value === 'string') {
            let sp = value.split(',');
            val = sp.map(x => Number(x));
            if (sp.length === 2) {
                this._initpoint = val;
            }
            if (sp.length === 3) {
                this._initpoint = [val[0], val[1]];
                this._initzoom = val[2];
            }
        }
    }

    get maxzoom() {
        return this._maxzoom;
    }
    set maxzoom(value) {
        if (typeof value === 'string') {
            this._maxzoom = Number(value);
        }
    }

    get minzoom() {
        return this._minzoom;
    }
    set minzoom(value) {
        if (typeof value === 'string') {
            this._minzoom = Number(value);
        }
    }

    calcModelParams() {
        let modelParams = { ...this._markerstyles }

        let dsParamsSet = new Set();
        let markerParamsSet = new Set();
        let modelParamsStruct = {};
        let markerParamsStruct = {};
        Object.entries(modelParams).forEach(val => {
            Object.keys(val[1]).forEach(key => {
                if (this._markerProps.includes(key)) {
                    markerParamsStruct[key] = val[1][key];
                    markerParamsSet.add(val[0]);
                } else {
                    modelParamsStruct[key.toLowerCase()] = val[1][key];
                    let fieldName = val[1][key] && val[1][key].FieldName;
                    dsParamsSet.add(fieldName);
                }
            })
        });
        this.modelParamsStruct = modelParamsStruct;
        this.markerParamsStruct = markerParamsStruct;
        this.dsParamsArray = Array.from(dsParamsSet);
        this.markerParamsArray = Array.from(markerParamsSet);
    }

    processDataSource(ds) {
        if (ds && Array.isArray(ds)) {
            if (this.needRecreate(ds)) {
                this.removeMarkers();
                this.createMarkers(ds)
            } else {
                this.updateDS(ds);
            }
        }
    }

    needRecreate(ds) {
        return this.currentMarkers.length === 0 || ds.length !== this.currentMarkers.length;
    }

    updateDS(ds) {
        const { toRecreate, toParamsUpdate } = this.compareDS(ds)

        if (toRecreate.length > 0) {
            let indexList = toRecreate.map(el => el.index);
            this.removeMarkers(indexList);
            this.createMarkers(toRecreate);
        }
        if (toParamsUpdate.length > 0) {
            toParamsUpdate.forEach(item => {
                let markerEntry = this.currentMarkers[item.index];
                if (typeof markerEntry !== "undefined") {
                    if (markerEntry.hasParams) {
                        let markWin = markerEntry.marker._icon.firstChild;
                        this.setWindParams(markWin, item);
                    }
                    if (markerEntry.popupOpened) {
                        this.setPopupParams(item.index);
                    }
                }
            })
        }
    }

    compareDS(ds) {
        let toRecreate = [], toParamsUpdate = [];
        let temp = [];
        ds.forEach((el, index) => {
            el.index = index;
            let markerEntry = this.currentMarkers[index];
            if (typeof markerEntry !== 'undefined') {
                let storedItem = markerEntry.item;
                if (storedItem && !this.compareEntry(el, storedItem, this.markerParamsArray)) {
                    toRecreate.push(el);
                } else {
                    temp.push(el);
                }
            }

        });
        temp.forEach(el => {
            let markerEntry = this.currentMarkers[el.index];
            if (typeof markerEntry !== 'undefined') {
                let storedItem = markerEntry.item;
                if (storedItem && !this.compareEntry(el, storedItem, this.dsParamsArray)) {
                    toParamsUpdate.push(el);
                }
            }
        });
        //then update current markers stored items
        ds.forEach(el => {
            this.currentMarkers[el.index].item = el
        })

        return {
            toRecreate,
            toParamsUpdate
        }
    }

    compareEntry(newItem, oldItem, keys = Object.keys(oldItem)) {
        // true if values are equal
        return keys.reduce((acc, cur) =>
            newItem[cur] == oldItem[cur] && acc, true);
    }

    createMarkers(ds) {
        let useIndexFlag = this._useindex;
        ds.forEach((item, ind) => {
            let markerKey;
            if (useIndexFlag) {
                let { index } = item;
                markerKey = index || ind;
            } else {
                markerKey = item.Id;
            }
            this.currentMarkers[markerKey] = this.createMarker(item, markerKey);
        });
    }

    removeMarkers(indexArray) {
        if (indexArray && indexArray.length > 0) {
            indexArray.forEach(ind => {
                let markerEntry = this.currentMarkers[ind];
                if (typeof markerEntry !== 'undefined') {
                    markerEntry.marker.removeFrom(this._Lmap);
                }
            })
        } else {
            this.currentMarkers.forEach(entry => {
                entry.marker.removeFrom(this._Lmap);
            })
            this.currentMarkers = [];
        }
    }


    getMarkerParam(key) {
        if (this._markerProps.includes(key) && this.markerParamsStruct[key]) {
            return this.markerParamsStruct[key]
        }
    }

    getMarkerParamDSValue(key, item, convert = true) {
        let struct = this.getMarkerParam(key);
        if (convert) {
            return Converter.convert(struct, item[struct.FieldName], struct.FieldType);
        } else {
            return item[struct.FieldName];
        }

    }

    getLocation(item) {
        let x, y;
        if (true/*настройка карты про формат координат*/) {
            x = this.getMarkerParamDSValue("X", item);
            y = this.getMarkerParamDSValue("Y", item);
        }
        return [y, x]; //lat long
        //значение y - lattitude - широта (значение 0 - экватор)
        //значение x - longitude - долгота (значение 0 - нулевой меридиан)

    }

    createMarker(item, markerKey) {
        let marker;
        let location = this.getLocation(item);
        let hasParams = false;
        let mTypeDef = this.getMarkerParamDSValue("MarkerType", item);
        if (mTypeDef.includes(".png") || mTypeDef.includes(".jpg")) { //TODO: detect icon format
            //resolve resource for marker picture
            let spl = mTypeDef.split(".");
            spl.splice(spl.length - 1, 1)
            let resName = spl.join('');
            let resource = `resources/${this.getResourceFromList(resName)}`;
            let icon = L.icon({
                iconUrl: resource,
                iconSize: [40, 40], //TODO: resolve size
            })
            marker = L.marker(location, { icon }).addTo(this._Lmap);
        }

        if (this._modelrefs && this._modelrefs[mTypeDef]) {
            let icon;
            let modelId = this._modelrefs[mTypeDef];
            let winModel = $WinDefs.winDef[modelId];

            if (winModel) {
                let windHtml = this.createWindow(winModel);
                icon = L.divIcon({
                    html: windHtml,
                    iconSize: [winModel.width, winModel.height],
                });
                marker = L.marker(location, { icon }).addTo(this._Lmap);
                let windNode = marker._icon.firstChild;
                hasParams = true;

                this.setWindParams(windNode, item);
                this.addTargets(windNode, markerKey, { item });
            }
        }

        if (typeof marker === 'undefined') {
            marker = L.marker(location).addTo(this._Lmap);
        }

        marker.on('click', this.markerOnClick.bind(this, markerKey));

        let tooltip = this.getMarkerParamDSValue("Tooltip", item);
        if (tooltip) {
            marker.bindTooltip(tooltip);
        }
        return { marker, item, hasParams };
    }

    createWindow(obj) {
        let param = "";
        const keys = Object.keys(obj);
        keys.forEach(key => {
            if (key !== 'windowTriggers') {
                param += ` ${key}="${obj[key]}"`;
            }
        });

        const winHtml = `<ms-window${param} position='absolute'></ms-window>`;
        //TODO: window triggers and check actions
        /*if (obj['windowTriggers']) {
            this._msWin.windowTriggers = JSON.parse(JSON.stringify(obj.windowTriggers));
        }*/

        return winHtml;
    }

    setWindParams(windRef, item) {
        if (typeof this.modelParamsStruct !== "undefined") {
            Object.entries(this.modelParamsStruct).forEach(val => {
                let key = val[0];
                let value = val[1];
                let fieldName = value && value.FieldName;
                if (windRef.hasAttribute(key)) {
                    let itemValue = Converter.convert(value, item[fieldName], value.FieldType);
                    windRef.SetParameter(key, itemValue, true);
                };
            });
        }
    }

    mapOnClick(ev) {
        let cusEv = new CustomEvent('mapclick', { detail: { x: ev.latlng.lng, y: ev.latlng.lat } })
        this.dispatchEvent(cusEv);
    }

    markerOnClick(id, ev) {
        let item = this.currentMarkers[id] && this.currentMarkers[id].item;
        let mtype = this.getMarkerParamDSValue("MarkerType", item, false);
        let cusEv = new CustomEvent('markerclick', { detail: { id, item, markerType: mtype, x: ev.latlng.lng, y: ev.latlng.lat } })
        this.dispatchEvent(cusEv);
    }

    openMarkerWindow(markerId, options) {
        let markerEntry = this.currentMarkers[markerId];
        if (typeof markerEntry !== 'undefined') {
            const winId = options.windowid;
            let winModel = $WinDefs.winDef[winId];
            let { popupOpened, popupInited } = markerEntry;
            if (!popupOpened && !popupInited && winModel && markerEntry.marker) {
                winModel.y = -winModel.height;
                let wind = this.createWindow(winModel);

                let opt = {
                    autoClose: options.closeonnewwindow,
                    closeOnClick: options.closeonlostfocus,
                    closeButton: options.canclose,
                };
                markerEntry.marker.bindPopup(wind, opt).openPopup();
                markerEntry.popupOpened = true;
                markerEntry.popupInited = true;
                this.setPopupParams(markerId);
                if (options.canclose) {
                    markerEntry.marker._popup._closeButton.style.left = Number(winModel.width) - 20;
                    markerEntry.marker._popup._closeButton.style.top = -winModel.height;
                    markerEntry.marker._popup._closeButton.style.zIndex = '10';
                }
                markerEntry.marker.on('popupopen', this.setPopupParams.bind(this, markerId));
                markerEntry.marker.on('popupclose', () => markerEntry.popupOpened = false);
            } else if (!popupOpened) {
                markerEntry.popupOpened = true;
            }
        }
    }

    setPopupParams(markerId) {
        let markerEntry = this.currentMarkers[markerId];
        if (typeof markerEntry !== 'undefined') {
            let msWin = markerEntry.marker._popup._contentNode.firstChild;
            this.setWindParams(msWin, markerEntry.item);
            this.setLinksPopup(markerEntry, markerId);
        }
    }

    setLinksPopup(markerEntry, markerId) {
        if (markerEntry.popupInited) {
            let msWin = markerEntry.marker._popup._contentNode.firstChild;
            this.addTargets(msWin, markerId, markerEntry);
        }
    }

    addTargets(msWin, markerId, markerEntry) {
        let map = this;
        let index = markerEntry.item.index || markerId;
        Object.keys(this.modelParamsStruct).forEach(key => {
            if (msWin.hasAttribute(key)) {
                if (typeof msWin.Links[key] !== 'undefined') {
                    msWin.Links[key].Targets.push({
                        el: map,
                        //HasBackward:false,
                        ItemId: map.id,
                        OneTime: false,
                        Operation: 'move',
                        PropertyPath: 'PanelDataSource',
                        Type: 'client',
                        ItemIndex: index,
                    });
                }
            }
        });
    }
}